﻿/*
 * Written by Andreas John Grech, 2011
 * http://blog.dreasgrech.com
 */

using System;
using System.Collections.Generic;
using System.IO;

namespace YouTubeUserUploads
{
    internal class Program
    {
        private static void Main(string[] args)
        {
            if (args.Length < 1)
            {
                Log(String.Format("Usage: {0} <username> [file]\n", AppDomain.CurrentDomain.FriendlyName), ConsoleColor.Red);
                return;
            }

            string username = args[0],
                   path = args.Length > 1 ? args[1] : Directory.GetCurrentDirectory() + @"\links.txt";

            Console.WriteLine("Username: " + username);
            Console.WriteLine("Saving links to: " + path);

            List<string> linkIDs;

            Log("\nProcessing...");

            try
            {
                linkIDs = UserUploadsLinkExtractor.RetrieveAll(username);
            }
            catch (Exception ex)
            {
                Log(ex.Message, ConsoleColor.Red);
                return;
            }

            if (linkIDs.Count == 0)
            {
                Log("User has no videos", ConsoleColor.Red);
                return;
            }

            WriteToFile(path, linkIDs);
        }

        private static void WriteToFile(string path, ICollection<string> linkIDs)
        {
            try
            {
                var tw = new StreamWriter(path);
                foreach (var linkID in linkIDs)
                {
                    tw.WriteLine("http://www.youtube.com/watch?v=" + linkID);
                }

                tw.Close();
            }
            catch (Exception ex)
            {
                Log(ex.Message, ConsoleColor.Red);
                return;
            }

            Log(String.Format("Finished writing {0} links", linkIDs.Count), ConsoleColor.Green);
        }

        private static void Log(string text = "", ConsoleColor color = ConsoleColor.Gray)
        {
            Console.ForegroundColor = color;
            Console.WriteLine(text);
            Console.ResetColor();
        }
    }
}