﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace YouTubeUserUploads
{
    static class UserUploadsLinkExtractor
    {
        private const int MAX_PER_PAGE = 50; // YouTube won't allow more than 50 at a time.

        public static List<string> RetrieveAll(string username)
        {
            var jobs = new List<Task<List<string>>>();
            var links = new List<string>();
            var totalUserUploads = GetNumberOfUserUploads(username);
            var roundTrips = (int)Math.Ceiling((double)totalUserUploads / MAX_PER_PAGE);

            for (var i = 0; i < roundTrips; i++)
            {
                var counter = i;
                jobs.Add(Task<List<string>>.Factory.StartNew(() =>
                                                                 {
                                                                     var startIndex = counter * MAX_PER_PAGE + 1;
                                                                     return GetUploads(username, startIndex);
                                                                 }));
            }

            Task.WaitAll(jobs.ToArray());

            return jobs.Aggregate(links, (current, job) => current.Concat(job.Result).ToList()); // Join the lists and return
        }

        static List<string> GetUploads(string username, int startIndex)
        {
            var url = UrlBuilder.UserUploadsURL(username, startIndex);
            var response = WebActions.ExecuteGETRequest(url);
            var strong = JSONSerializer.Deserialise<UserUploadsResponseRepresentation>(response);
            var ids = ExtractIDs(strong);

            return ids.ToList();
        }

        static int GetNumberOfUserUploads(string username)
        {
            var url = UrlBuilder.UserInfo(username);
            var response = WebActions.ExecuteGETRequest(url);
            if (String.IsNullOrEmpty(response))
            {
                throw new Exception("That user does not exist");
            }

            var strong = JSONSerializer.Deserialise<UserInfoResponseRepresentation>(response);

            return Convert.ToInt32(strong.Entry.FeedLink.First(f => f.Rel == "http://gdata.youtube.com/schemas/2007#user.uploads").CountHint);
        }

        static IEnumerable<string> ExtractIDs(UserUploadsResponseRepresentation uploadsResponseRepresentation)
        {
            var entries = uploadsResponseRepresentation.Feed.Entry;
            return entries.Select(entryResponseRepresentation => entryResponseRepresentation.ID.T).Select(id => id.Substring(id.LastIndexOf("/") + 1));
        }
    }
}
